# This file is part of the Frescobaldi project, http://www.frescobaldi.org/
#
# Copyright (c) 2008 - 2012 by Wilbert Berendsen
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# See http://www.gnu.org/licenses/ for more information.

"""
Dummy file containing messages that are part of some Python modules or Qt and used in our application.
"""

from __future__ import unicode_literals

# Python, optparse:
_("Usage")
_("Usage: %s\n")
_("show this help message and exit")
_("show program's version number and exit")
# L10N: Command line options
_("Options")


# Qt
_("CloseButton", "Close Tab")

_("QDialog", "What's This?")

_("QDialogButtonBox", "Apply")
_("QDialogButtonBox", "OK")
_("QDialogButtonBox", "Cancel")
_("QDialogButtonBox", "Discard")
_("QDialogButtonBox", "Save")
_("QDialogButtonBox", "Help")
_("QDialogButtonBox", "Close")
_("QDialogButtonBox", "Open")
_("QDialogButtonBox", "Reset")

_("QFileDialog", "File")
_("QFileDialog", "Folder")
_("QFileDialog", "Find Directory")
_("QFileDialog", "All Files (*)")

_("QScrollBar", "Scroll here")
_("QScrollBar", "Top")
_("QScrollBar", "Bottom")
_("QScrollBar", "Page up")
_("QScrollBar", "Page down")
_("QScrollBar", "Scroll up")
_("QScrollBar", "Scroll down")
_("QScrollBar", "Left edge")
_("QScrollBar", "Right edge")
_("QScrollBar", "Page left")
_("QScrollBar", "Page right")
_("QScrollBar", "Scroll left")
_("QScrollBar", "Scroll right")

